@@ -151,23 +151,31 @@ class Quantizer(nn.Module):
 class SignedQuantizer(Quantizer):
     def __init__(self, *args, **kwargs):
         super(SignedQuantizer, self).__init__(*args, **kwargs)
-        self.quant_min_val = torch.tensor((-(1 << (self.bits - 1))), device=self.observer.device)
-        self.quant_max_val = torch.tensor(((1 << (self.bits - 1)) - 1), device=self.observer.device)
+        if self.activation_weight_flag == 0: # weight
+            self.quant_min_val = torch.tensor((-((1 << (self.bits - 1)) - 1)), device=self.observer.device)
+            self.quant_max_val = torch.tensor(((1 << (self.bits - 1)) - 1), device=self.observer.device)
+        elif self.activation_weight_flag == 1: # activation
+            self.quant_min_val = torch.tensor((-(1 << (self.bits - 1))), device=self.observer.device)
+            self.quant_max_val = torch.tensor(((1 << (self.bits - 1)) - 1), device=self.observer.device)
+        else:
+            print('activation_weight_flag error')
 
 class UnsignedQuantizer(Quantizer):
     def __init__(self, *args, **kwargs):
         super(UnsignedQuantizer, self).__init__(*args, **kwargs)
-        self.quant_min_val = torch.tensor((0), device=self.observer.device)
-        self.quant_max_val = torch.tensor(((1 << self.bits) - 1), device=self.observer.device)
+        if self.activation_weight_flag == 0: # weight:
+            self.quant_min_val = torch.tensor((0), device=self.observer.device)
+            self.quant_max_val = torch.tensor(((1 << self.bits) - 2), device=self.observer.device)
+        elif self.activation_weight_flag == 1: # activation
+            self.quant_min_val = torch.tensor((0), device=self.observer.device)
+            self.quant_max_val = torch.tensor(((1 << self.bits) - 1), device=self.observer.device)
+        else:
+            print('activation_weight_flag error')
 
 # 对称量化
 class SymmetricQuantizer(SignedQuantizer):
     def update_qparams(self):
-        # quantized_range
-        if self.activation_weight_flag == 0: 
-            quant_range = float(torch.min(torch.abs(self.quant_min_val), torch.abs(self.quant_max_val)))    # weight               
-        else:                              
-            quant_range = float(self.quant_max_val - self.quant_min_val) / 2                                # activation
+        quant_range = float(self.quant_max_val - self.quant_min_val) / 2                                # quantized_range
         float_range = torch.max(torch.abs(self.observer.min_val), torch.abs(self.observer.max_val))     # float_range
         self.scale = float_range / quant_range                                                          # scale
         self.scale = torch.max(self.scale, self.eps)                                                    # processing for very small scale
@@ -224,21 +232,21 @@ class QuantConv2d(nn.Conv2d):
                                                                q_level='L', out_channels=None, device=device), activation_weight_flag=0)
         else:
             self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(
-                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=2)
+                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=1)
             if weight_observer == 0:
                 if q_level == 0:
                     self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(
-                                                                q_level='C', out_channels=out_channels, device=device), activation_weight_flag=2)
+                                                                q_level='C', out_channels=out_channels, device=device), activation_weight_flag=0)
                 else:
                     self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(
-                                                                q_level='L', out_channels=None, device=device), activation_weight_flag=2)
+                                                                q_level='L', out_channels=None, device=device), activation_weight_flag=0)
             else:
                 if q_level == 0:
                     self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(
-                                                                q_level='C', out_channels=out_channels, device=device), activation_weight_flag=2)
+                                                                q_level='C', out_channels=out_channels, device=device), activation_weight_flag=0)
                 else:
                     self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(
-                                                                q_level='L', out_channels=None, device=device), activation_weight_flag=2)
+                                                                q_level='L', out_channels=None, device=device), activation_weight_flag=0)
 
     def forward(self, input):
         quant_input = self.activation_quantizer(input)
@@ -259,9 +267,9 @@ class QuantConvTranspose2d(nn.ConvTranspose2d):
                  stride=1,
                  padding=0,
                  output_padding=0,
-                 dilation=1,
                  groups=1,
                  bias=True,
+                 dilation=1,
                  padding_mode='zeros',
                  a_bits=8,
                  w_bits=8,
@@ -270,7 +278,7 @@ class QuantConvTranspose2d(nn.ConvTranspose2d):
                  weight_observer=0,
                  quant_inference=False):
         super(QuantConvTranspose2d, self).__init__(in_channels, out_channels, kernel_size, stride, padding, output_padding,
-                                                   dilation, groups, bias, padding_mode)
+                                                   groups, bias, dilation, padding_mode)
         self.quant_inference = quant_inference
         if q_type == 0:
             self.activation_quantizer = SymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(
@@ -283,13 +291,13 @@ class QuantConvTranspose2d(nn.ConvTranspose2d):
                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=0)
         else:
             self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(
-                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=2)
+                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=1)
             if weight_observer == 0:
                 self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(
-                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=2)
+                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=0)
             else:
                 self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(
-                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=2)
+                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=0)
 
     def forward(self, input):
         quant_input = self.activation_quantizer(input)
@@ -327,10 +335,12 @@ class QuantBNFuseConv2d(QuantConv2d):
                  q_type=0,
                  q_level=0,
                  device='cpu',
-                 weight_observer=0):
+                 weight_observer=0,
+                 pretrained_model=False):
         super(QuantBNFuseConv2d, self).__init__(in_channels, out_channels, kernel_size, stride, padding, dilation, groups,
                                                 bias, padding_mode)
         self.num_flag = 0
+        self.pretrained_model = pretrained_model
         self.eps = eps
         self.momentum = momentum
         self.gamma = Parameter(torch.Tensor(out_channels))
@@ -359,21 +369,21 @@ class QuantBNFuseConv2d(QuantConv2d):
                                                                q_level='L', out_channels=None, device=device), activation_weight_flag=0)
         else:
             self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(
-                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=2)
+                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=1)
             if weight_observer == 0:
                 if q_level == 0:
                     self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(
-                                                                q_level='C', out_channels=out_channels, device=device), activation_weight_flag=2)
+                                                                q_level='C', out_channels=out_channels, device=device), activation_weight_flag=0)
                 else:
                     self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(
-                                                                q_level='L', out_channels=None, device=device), activation_weight_flag=2)
+                                                                q_level='L', out_channels=None, device=device), activation_weight_flag=0)
             else:
                 if q_level == 0:
                     self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(
-                                                                q_level='C', out_channels=out_channels, device=device), activation_weight_flag=2)
+                                                                q_level='C', out_channels=out_channels, device=device), activation_weight_flag=0)
                 else:
                     self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(
-                                                                q_level='L', out_channels=None, device=device), activation_weight_flag=2)
+                                                                q_level='L', out_channels=None, device=device), activation_weight_flag=0)
 
     def forward(self, input):
         # 训练态
@@ -386,15 +396,21 @@ class QuantBNFuseConv2d(QuantConv2d):
             batch_mean = torch.mean(output, dim=dims)
             batch_var = torch.var(output, dim=dims)
             with torch.no_grad():
-                if self.num_flag == 0:
-                    self.num_flag += 1
-                    running_mean = batch_mean
-                    running_var = batch_var
+                if not self.pretrained_model:
+                    if self.num_flag == 0:
+                        self.num_flag += 1
+                        running_mean = batch_mean
+                        running_var = batch_var
+                    else:
+                        running_mean = (1 - self.momentum) * self.running_mean + self.momentum * batch_mean
+                        running_var = (1 - self.momentum) * self.running_var + self.momentum * batch_var
+                    self.running_mean.copy_(running_mean)
+                    self.running_var.copy_(running_var)
                 else:
                     running_mean = (1 - self.momentum) * self.running_mean + self.momentum * batch_mean
                     running_var = (1 - self.momentum) * self.running_var + self.momentum * batch_var
-                self.running_mean.copy_(running_mean)
-                self.running_var.copy_(running_var)
+                    self.running_mean.copy_(running_mean)
+                    self.running_var.copy_(running_var)
             # BN融合
             if self.bias is not None:
                 bias_fused = reshape_to_bias(self.beta + (self.bias - batch_mean) * (self.gamma / torch.sqrt(batch_var + self.eps)))
@@ -459,21 +475,21 @@ class QuantLinear(nn.Linear):
                                                                q_level='L', out_channels=None, device=device), activation_weight_flag=0)
         else:
             self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(
-                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=2)
+                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=1)
             if weight_observer == 0:
                 if q_level == 0:
                     self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(
-                                                                q_level='FC', out_channels=out_features, device=device), activation_weight_flag=2)
+                                                                q_level='FC', out_channels=out_features, device=device), activation_weight_flag=0)
                 else:
                     self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MinMaxObserver(
-                                                                q_level='L', out_channels=None, device=device), activation_weight_flag=2)
+                                                                q_level='L', out_channels=None, device=device), activation_weight_flag=0)
             else:
                 if q_level == 0:
                     self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(
-                                                                q_level='FC', out_channels=out_features, device=device), activation_weight_flag=2)
+                                                                q_level='FC', out_channels=out_features, device=device), activation_weight_flag=0)
                 else:
                     self.weight_quantizer = AsymmetricQuantizer(bits=w_bits, observer=MovingAverageMinMaxObserver(
-                                                                q_level='L', out_channels=None, device=device), activation_weight_flag=2)
+                                                                q_level='L', out_channels=None, device=device), activation_weight_flag=0)
 
     def forward(self, input):
         quant_input = self.activation_quantizer(input)
@@ -493,7 +509,7 @@ class QuantReLU(nn.ReLU):
                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=1)
         else:
             self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(
-                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=2)
+                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=1)
 
     def forward(self, input):
         quant_input = self.activation_quantizer(input)
@@ -509,7 +525,7 @@ class QuantSigmoid(nn.Sigmoid):
                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=1)
         else:
             self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(
-                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=2)
+                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=1)
 
     def forward(self, input):
         quant_input = self.activation_quantizer(input)
@@ -527,7 +543,7 @@ class QuantMaxPool2d(nn.MaxPool2d):
                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=1)
         else:
             self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(
-                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=2)
+                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=1)
 
     def forward(self, input):
         quant_input = self.activation_quantizer(input)
@@ -546,7 +562,7 @@ class QuantAvgPool2d(nn.AvgPool2d):
                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=1)
         else:
             self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(
-                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=2)
+                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=1)
 
     def forward(self, input):
         quant_input = self.activation_quantizer(input)
@@ -563,7 +579,7 @@ class QuantAdaptiveAvgPool2d(nn.AdaptiveAvgPool2d):
                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=1)
         else:
             self.activation_quantizer = AsymmetricQuantizer(bits=a_bits, observer=MovingAverageMinMaxObserver(
-                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=2)
+                                                            q_level='L', out_channels=None, device=device), activation_weight_flag=1)
 
     def forward(self, input):
         quant_input = self.activation_quantizer(input)
@@ -572,7 +588,7 @@ class QuantAdaptiveAvgPool2d(nn.AdaptiveAvgPool2d):
 
 
 def add_quant_op(module, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu',
-                 weight_observer=0, bn_fuse=0, quant_inference=False):
+                 weight_observer=0, bn_fuse=0, quant_inference=False, pretrained_model=False):
     for name, child in module.named_children():
         if isinstance(child, nn.Conv2d):
             if bn_fuse:
@@ -617,7 +633,8 @@ def add_quant_op(module, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu',
                                                            q_type=q_type,
                                                            q_level=q_level,
                                                            device=device,
-                                                           weight_observer=weight_observer)
+                                                           weight_observer=weight_observer,
+                                                           pretrained_model=pretrained_model)
                     quant_bn_fuse_conv.bias.data = conv_child_temp.bias
                 else:
                     quant_bn_fuse_conv = QuantBNFuseConv2d(conv_child_temp.in_channels,
@@ -636,7 +653,8 @@ def add_quant_op(module, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu',
                                                            q_type=q_type,
                                                            q_level=q_level,
                                                            device=device,
-                                                           weight_observer=weight_observer)
+                                                           weight_observer=weight_observer,
+                                                           pretrained_model=pretrained_model)
                 quant_bn_fuse_conv.weight.data = conv_child_temp.weight
                 quant_bn_fuse_conv.gamma.data = child.weight
                 quant_bn_fuse_conv.beta.data = child.bias
@@ -653,9 +671,9 @@ def add_quant_op(module, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu',
                                                             stride=child.stride,
                                                             padding=child.padding,
                                                             output_padding=child.output_padding,
-                                                            dilation=child.dilation,
                                                             groups=child.groups,
                                                             bias=True,
+                                                            dilation=child.dilation,
                                                             padding_mode=child.padding_mode,
                                                             a_bits=a_bits,
                                                             w_bits=w_bits,
@@ -671,9 +689,9 @@ def add_quant_op(module, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu',
                                                             stride=child.stride,
                                                             padding=child.padding,
                                                             output_padding=child.output_padding,
-                                                            dilation=child.dilation,
                                                             groups=child.groups,
                                                             bias=False,
+                                                            dilation=child.dilation,
                                                             padding_mode=child.padding_mode,
                                                             a_bits=a_bits,
                                                             w_bits=w_bits,
@@ -730,18 +748,18 @@ def add_quant_op(module, a_bits=8, w_bits=8, q_type=0, q_level=0, device='cpu',
                                                              device=device)
             module._modules[name] = quant_adaptive_avg_pool
         else:
-            add_quant_op(child, a_bits=a_bits, w_bits=w_bits, q_type=q_type,
-                         q_level=q_level, device=device, weight_observer=weight_observer,
-                         bn_fuse=bn_fuse, quant_inference=quant_inference)
+            add_quant_op(child, a_bits=a_bits, w_bits=w_bits, q_type=q_type, q_level=q_level,
+                         device=device, weight_observer=weight_observer, bn_fuse=bn_fuse,
+                         quant_inference=quant_inference, pretrained_model=pretrained_model)
 
 
 def prepare(model, inplace=False, a_bits=8, w_bits=8, q_type=0, q_level=0,
-            device='cpu', weight_observer=0, bn_fuse=0, quant_inference=False):
+            device='cpu', weight_observer=0, bn_fuse=0, quant_inference=False, pretrained_model=False):
     if not inplace:
         model = copy.deepcopy(model)
     add_quant_op(model, a_bits=a_bits, w_bits=w_bits, q_type=q_type, q_level=q_level,
                  device=device, weight_observer=weight_observer, bn_fuse=bn_fuse,
-                 quant_inference=quant_inference)
+                 quant_inference=quant_inference, pretrained_model=pretrained_model)
     return model
 
 
